/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.annotation.SuppressLint
import android.graphics.drawable.TransitionDrawable
import android.view.MotionEvent
import com.commonsense.android.kotlin.views.extensions.gone
import com.commonsense.android.kotlin.views.extensions.visibleOrGone
import dk.nsp.domain.knowledge.ArticleContentSection
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.ArticleParagraphCardRowBinding
import dk.nsp.mingraviditet.extensions.disable
import dk.nsp.mingraviditet.extensions.visibleOnTrue
import dk.nsp.mingraviditet.util.android.OpenExternalLinksInBrowserWebViewClient
import dk.nsp.mingraviditet.util.android.WebViewHelper
import dk.nsp.mingraviditet.util.android.WebViewHelper.setUpDarkModeSupport
import dk.nsp.mingraviditet.util.android.recycler.BindingViewHolder
import dk.nsp.mingraviditet.util.android.recycler.DelegationListAdapterInflater
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.bindingadapters.isTextEllipsized
import dk.nsp.shared.util.launchDefault
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import timber.log.Timber


class ArticleParagraphCardRow(
    val model: ArticleContentSection,
    val relevantId: Int
) :
    RecyclerAdapterView<ArticleParagraphCardRowBinding> {

    @SuppressLint("SetJavaScriptEnabled", "ClickableViewAccessibility")
    override fun bindData(viewHolder: BindingViewHolder<ArticleParagraphCardRowBinding>) {
        viewHolder.binding.articleContent = model

        viewHolder.binding.webview.webViewClient =
            OpenExternalLinksInBrowserWebViewClient(viewHolder.binding.root.context)

        viewHolder.binding.webview.apply {
            setUpDarkModeSupport()

            // Enable js to be able to show videoes
            settings.javaScriptEnabled = true
            settings.useWideViewPort = false
            settings.loadWithOverviewMode = false

            // Disable scroll on touch
            disable()

            val htmlToLoad = WebViewHelper.createHtmlWithStyleSheet(
                model.content,
                viewHolder.binding.root.context
            )
            loadDataWithBaseURL("file:///android_asset/", htmlToLoad, "text/html", "UTF-8", "")
        }

        // Show small animation when coming from search
        if (relevantId == model.id) {
            val transition = viewHolder.binding.rowBackground.background as TransitionDrawable

            CoroutineScope(Dispatchers.Main).launchDefault {
                transition.startTransition(250)
                delay(500)
                transition.reverseTransition(500)
            }

            toggleExpandCollapseState(viewHolder.binding)
        } else {
            // Otherwise ensure that the card isExpanded state matches current models value
            updateForNewExpandedState(viewHolder.binding)
        }

        // Always default button to gone, otherwise it can flicker on fast scrolling
        viewHolder.binding.expandCollapseButton.gone()
        viewHolder.binding.expandCollapseButton.setOnClickListener {
            toggleExpandCollapseState(viewHolder.binding)
        }

        viewHolder.binding.executePendingBindings()
        viewHolder.binding.contentBody.post {
            viewHolder.binding.expandCollapseButton.visibleOnTrue(
                viewHolder.binding.contentBody.isTextEllipsized()
                    ||  model.isExpanded
                        || viewHolder.binding.contentBody.text.isBlank()
            )
        }
    }

    private fun toggleExpandCollapseState(binding: ArticleParagraphCardRowBinding) {
        model.isExpanded = !model.isExpanded
        updateForNewExpandedState(binding)
    }

    private fun updateForNewExpandedState(binding: ArticleParagraphCardRowBinding) {
        binding.webview.visibleOrGone(model.isExpanded)
        binding.contentBody.visibleOrGone(!model.isExpanded)

        if (model.isExpanded) {
            binding.expandCollapseButton.setText(R.string.common_cta_collapse)
        } else {
            binding.expandCollapseButton.setText(R.string.common_cta_expand)
        }
    }


    override val createHolderFunction: DelegationListAdapterInflater<ArticleParagraphCardRowBinding> =
        ArticleParagraphCardRowBinding::inflate

    override fun isContentTheSameAsInternal(other: RecyclerAdapterView<*>): Boolean = when(other) {
        is ArticleParagraphCardRow -> other.model == model && other.relevantId == relevantId
        else -> false
    }
}