/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.util

import android.os.Build
import dk.nsp.domain.knowledge.Article
import dk.nsp.domain.knowledge.ArticleCategory
import dk.nsp.domain.knowledge.ArticleContentSection
import dk.nsp.testassertions.assertEquals
import dk.nsp.testassertions.assertFalse
import dk.nsp.testassertions.assertNotNullAndEquals
import dk.nsp.testassertions.assertTrue
import org.junit.runner.RunWith
import org.robolectric.RobolectricTestRunner
import org.robolectric.annotation.Config
import java.time.LocalDateTime

@Config(sdk = [Build.VERSION_CODES.P], manifest = Config.NONE)
@RunWith(RobolectricTestRunner::class)
class ArticleCategoryExtensionsTest {

    private val emptyModel = ArticleCategory(
        "",
        "",
        null,
        emptyList(),
        ""
    )

    private val withSingleArticle = ArticleCategory(
        "",
        "",
        null,
        listOf(
            Article(
                1,
                "Title1",
                1,
                LocalDateTime.MIN,
                "Description1",
                listOf(
                    ArticleContentSection(
                        1,
                        "Title1",
                        1,
                        "Content1",
                        listOf("week1")
                    )
                )
            )
        ),
        ""
    )

    private val withThreeArticles = ArticleCategory(
        "",
        "",
        null,
        listOf(
            Article(
                31,
                "Title31",
                1,
                LocalDateTime.MIN,
                "Description31",
                listOf(
                    ArticleContentSection(
                        311,
                        "Title311",
                        null,
                        "Content311",
                        listOf("week311")
                    )
                )
            ),
            Article(
                32,
                "Title32",
                2,
                LocalDateTime.MIN,
                "Description32",
                listOf(
                    ArticleContentSection(
                        321,
                        "Title321",
                        4,
                        "Content321",
                        listOf("week321")
                    ),
                    ArticleContentSection(
                        322,
                        "Title322",
                        3,
                        "Content322",
                        listOf("week322")
                    )
                )
            ),
            Article(
                33,
                "Title33",
                5,
                LocalDateTime.MIN,
                "Description33",
                emptyList()
            )
        ),
        ""
    )

    @org.junit.Test
    fun articleCategorySearchResults() {
        val searchResults = withThreeArticles.searchResults()

        searchResults.size.assertEquals(5)

        // ArticleContentSection with id 311 is skipped due to searchOrder = null
        searchResults.getOrNull(0)?.title.assertNotNullAndEquals("Title31")
        searchResults.getOrNull(1)?.title.assertNotNullAndEquals("Title32")
        searchResults.getOrNull(2)?.title.assertNotNullAndEquals("Title322")
        searchResults.getOrNull(3)?.title.assertNotNullAndEquals("Title321")
        searchResults.getOrNull(4)?.title.assertNotNullAndEquals("Title33")
    }

    @org.junit.Test
    fun hasSearchResultsEmpty() {
        emptyList<ArticleCategory>().hasSearchResults().assertFalse()
    }

    @org.junit.Test
    fun hasSearchResultsSingle() {
        val emptyContents = emptyModel.copy(
            articles = listOf(
                Article(
                    1,
                    "Title",
                    null,
                    LocalDateTime.MIN,
                    "Description",
                    emptyList()
                )
            )
        )
        listOf(emptyContents).hasSearchResults().assertFalse()

        listOf(withSingleArticle).hasSearchResults().assertTrue()
    }

    @org.junit.jupiter.api.Test
    fun articleCategoryIsAfterBirth() {
        emptyModel.isAfterBirth.assertFalse()
        emptyModel.copy(slug = "efterfødsel").isAfterBirth.assertFalse()
        emptyModel.copy(slug = "slug").isAfterBirth.assertFalse()
        emptyModel.copy(slug = "Efterfoedsel").isAfterBirth.assertFalse()

        emptyModel.copy(slug = "efterfoedsel").isAfterBirth.assertTrue()
    }
}